"""Summary.

Attributes:
    VERSION (str): Description

"""
VERSION = '0.3.24-alpha'


def get_version() -> str:
    """Summary.

    Returns:
        str: Description

    """
    return VERSION


if __name__ == '__main__':
    print(get_version())
